/***************************************************************************
 * Game.java - The JavaChess game controller
 * by Franois Dominic Larame
 *
 * Purpose: Coordinate the efforts of all other game-related objects.  This
 * work has been separated from the App (application-level object) because
 * the latter may come to take on more message-passing and related duties
 * if I ever add a GUI to the game, and I didn't want interface and game
 * mechanics to get mixed up in a single class.
 *
 * History
 * 08.06.00 Created
 **************************************************************************/
package javachess;

import javachess.Player;
import javachess.Board;
import javachess.OpeningBook;
import java.io.*;

/**************************************************************************
 * public class Game
 *************************************************************************/

public class Game
{
  // The two players involved in the current game
  Player Players[];

  // The state of the game
  Board GameBoard;

  // The opening book
  OpeningBook Openings;

  // A wrapper for the keyboard
  InputStreamReader kbd;

  // Constructor
  public Game()
  {
  }

  // boolean InitializeGame()
  // Select the players, create subsidiary objects and prepare to play
  public boolean InitializeGame( String openingBook, String startingPos ) throws Exception
  {
    // Read the opening book
    Openings = new OpeningBook();
    Openings.Load( openingBook );

    // Load the initial position, if any
    GameBoard = new Board();
    if ( startingPos.equalsIgnoreCase( "NONE" ) )
      	GameBoard.StartingBoard();
    else
      GameBoard.Load( startingPos );

    // Initialize the keyboard
    kbd = new InputStreamReader( System.in );
    int key;

    // Identify the two players
    Players = new Player[ 2 ];
//    key = 'C';
//    System.out.println( "Welcome to Java Chess.  Who plays white: [H]uman or [C]omputer? " );
//    try {
//      do
//      {
//        key = kbd.read();
//      }
//      while( ( key != 'H' ) && ( key != 'h' ) && ( key != 'c' ) && ( key != 'C' ) );
//    } catch( IOException e ) {}
//
//    if ( ( key == 'H' ) || ( key == 'h' ) )
//    {
//      Players[ Player.SIDE_WHITE ] = new PlayerHuman( Player.SIDE_WHITE, kbd );
//    }
//    else
//    {
//      Players[ Player.SIDE_WHITE ] =
//        new PlayerAI( Player.SIDE_WHITE, AISearchAgent.AISEARCH_MTDF, Openings );
//    }
//
//    System.out.println( "And who plays black: [H]uman or [C]omputer? " );
//    try {
//      do
//      {
//        key = kbd.read();
//      }
//      while( ( key != 'H' ) && ( key != 'h' ) && ( key != 'c' ) && ( key != 'C' ) );
//    } catch( IOException e ) {}
//
//    if ( ( key == 'H' ) || ( key == 'h' ) )
//    {
//      Players[ Player.SIDE_BLACK ] = new PlayerHuman( Player.SIDE_BLACK, kbd );
//    }
//    else
//    {
//      Players[ Player.SIDE_BLACK ] =
//        new PlayerAI( Player.SIDE_BLACK, AISearchAgent.AISEARCH_MTDF, Openings );
//    }
	
	System.out.println( "Welcome to Java Chess.");
	Players[ Player.SIDE_WHITE ] = new PlayerHuman( Player.SIDE_WHITE, kbd );
	Players[ Player.SIDE_BLACK ] = new PlayerAI( Player.SIDE_BLACK, AISearchAgent.AISEARCH_MTDF, Openings );
    return true;
  }

  // boolean RunGame()
  // A simple loop getting moves from the current player until the game is over
  public boolean RunGame() throws Exception
  {
    Player CurrentPlayer;
    Move Mov;

    do
    {
      // Show the current game board
      GameBoard.Print();

      // Ask the next player for a move
      CurrentPlayer = Players[ GameBoard.GetCurrentPlayer() ];
      Mov = CurrentPlayer.GetMove( GameBoard );
      System.out.print( Player.PlayerStrings[ GameBoard.GetCurrentPlayer() ] );
      System.out.print( " selects move: " );
      Mov.Print();

      // Change the state of the game accordingly
      GameBoard.ApplyMove( Mov );

      // Pause
      //Thread.currentThread().sleep( 2000 );

    } while( ( Mov.MoveType != Move.MOVE_RESIGN ) &&
             ( Mov.MoveType != Move.MOVE_STALEMATE ) );

    System.out.println( "Game Over.  Thanks for playing!" ) ;

    return true;
  }
}