/**************************************************************************
 *
 * PlayerAI.java - Interface to a computer player
 * by Franois Dominic Larame
 *
 * Purpose: This object allows a computer player to play JavaChess.  Its only
 * real job is to query an AI Search Agent for his move.
 *
 * History:
 * 11.06.00 Creation
 * 07.08.00 Association with the search agent
 *
 **************************************************************************/

package javachess;
import javachess.AISearchAgent;

public class PlayerAI extends Player
{
  /************************************************************************
   * DATA MEMBERS
   ***********************************************************************/

  // The search agent in charge of the moves
  AISearchAgent Agent;

  /***********************************************************************
   * PUBLIC METHODS
   **********************************************************************/

  // Constructor
  public PlayerAI( int whichPlayer, int whichType, OpeningBook ref )
  {
    this.SetSide( whichPlayer );
    Agent = AISearchAgent.MakeNewAgent( whichType, ref );
  }

  // Attach a search agent to the AI player
  public boolean AttachSearchAgent( AISearchAgent theAgent )
  {
    Agent = theAgent;
    return true;
  }

  // Getting a move from the machine
  public Move GetMove( Board theBoard )
  {
    return( Agent.PickBestMove( theBoard ) );
  }
}