/*
   Copyright (C) 2001 Alejandro Dubrovsky
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License

 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "common.h"
#include "search.h"
#include "eval.h"
#include "hash.h"
#include "utils.h"
#include "board.h"
#include "pieces.h"
#include "ai.h"
#include <libale.h>
#include <sys/time.h>


/* MTD(f) - straight out of "MTD(f), a new chess algorithm" by Aske Plaat */
int MTDf(Board * b, Move * move,
					 int depth, int value, Flags *flags)
{

	int upperbound, lowerbound, beta, calls, gottime;
	struct timeval timestarted, timefinished, temptime;
	char buffer[20];
	
	calls = 0;
	gottime = 1;
	upperbound = MAXVALUE;
	lowerbound = MINVALUE;
	
	b->lastcheck = b->totalnodes;	
	do {

		if (b->verbosity > 8) {
			gettimeofday(&timestarted, NULL);
		}
		calls ++;
		if (value == lowerbound) {
			beta = value + 1;
		} else {
			beta = value;
		}
		flags->signal |= FLAG_SIGNAL_USEMOVE | FLAG_SIGNAL_DONTHASH;
		value = veryNewAlphaBeta(b, move, depth, beta - 1, beta,flags);
		if (flags->signal & FLAG_SIGNAL_OUTOFTIME) {
			gottime = 0;
		}
		if (value < beta) {
			upperbound = value;
		} else {
			lowerbound = value;
		}
		if (b->verbosity > 8) {
			moveToSan(b,move,buffer);
			gettimeofday(&timefinished, NULL);			
			timersub(&timefinished, &timestarted, &temptime);			
			fprintf(b->logfile,"MTDF - Depth %d, call %d, difference %d best move %s\n",
			depth,calls, upperbound - lowerbound, buffer);
			fprintf(b->logfile,
			  "Call took %ld.%06ld seconds\n",
			  (long int) temptime.tv_sec, (long int) temptime.tv_usec);

		}
		if ((b->totalnodes > b->lastcheck + NODES_PER_CHECK) && (lowerbound >= upperbound) && (gottime)) {
			b->lastcheck = b->totalnodes;
			gettimeofday(&temptime,NULL);
			if ((timercmp(&temptime,&(flags->stoptime),>))  && (!(b->actionmode & ANALYSEMODE))) {
				flags->signal |= FLAG_SIGNAL_OUTOFTIME;
				gottime = 0;
			}
		}
	} while ((gottime) && (lowerbound < upperbound));
	
	if (b->verbosity > 5) {
		fprintf(b->logfile,"MTDF - At depth %d, called %d times\n",depth,calls);
		if (!gottime) {
			fprintf(b->logfile,"MTDF - out of time\n");
		}
	}
	return value;
}



