// AISearchAgent.h: Schnittstelle fr die Klasse AISearchAgent.
//
//////////////////////////////////////////////////////////////////////

#if !defined( AFX_AISEARCHAGENT_H__B1DEBB74_4ACA_4FC8_ADEB_CDA8861DED4B__INCLUDED_ )
#define AFX_AISEARCHAGENT_H__B1DEBB74_4ACA_4FC8_ADEB_CDA8861DED4B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "TranspositionTable.h"
#include "CutoffHistory.h"
#include "BoardEvaluator.h"
#include "OpeningBook.h"


class AISearchAgent  
{
public:
  // ID's for concrete subclasses; AISearchAgent works as a factory for its
  // concrete subclasses
  static const int AISEARCH_ALPHABETA;
  static const int AISEARCH_MTDF;
  static const int AISEARCH_NEGASCOUT;

  static const int ALPHABETA_MAXVAL;
  static const int ALPHABETA_MINVAL;

protected:
  // Alphabeta search boundaries
  static const int ALPHABETA_ILLEGAL;
  /**
   * A score below which we give up: if Alphabeta ever returns a value lower
   * than this threshold, then all is lost and we might as well resign.  Here,
   * the value is equivalent to "mated by the opponent in 3 moves or less".
   */
  static const int ALPHABETA_GIVEUP;

  // Alphabeta search cutoff types
  static const int ALPHABETA_NOCUT;
  static const int ALPHABETA_BETACUT;
  static const int ALPHABETA_BELOWALPHA;
  static const int ALPHABETA_LOWCUT;
  static const int ALPHABETA_HIGHCUT;

  /** An approximate upper bound on the total value of all positional
   * terms in the evaluation function
   */
  static const int EVAL_THRESHOLD;
	/** 
   * Search node types: MAXNODEs are nodes where the computer player is the
   * one to move
   */
  static const bool MAXNODE;
  /** 
   * Search node types: MINNODEs are positions where the opponent is to move.
   */
  static const bool MINNODE;

public:
	int NumRegularNodes;
  /** A transposition table for this object */
	TranspositionTable TransTable;
	TranspositionTable TransTableBlack;
	TranspositionTable **TransTables;
protected:

  /** How will we assess position strengths? */
  BoardEvaluator* Evaluator;

  int FromWhosePerspective;

  // Statistics
  int NumQuiescenceNodes;
  int NumRegularTTHits;
  int NumQuiescenceTTHits;
  int NumRegularCutoffs;
  int NumQuiescenceCutoffs;
  int NumRegularHashCollisions;
  int NumQuiescenceHashCollisions;
  int NumRegularMoves;
  int NumUsedRegularMoves;
  int NumQuiescenceMoves;
  int NumUsedQuiescenceMoves;

  /**
   *  A move counter, so that the agent knows when it can delete old stuff from
   *  its transposition table
   */
  int MoveCounter;
  CutoffHistory CutoffHist;
  CutoffHistory qCutoffHist;


/***********************************************************************/
/*                             Operationen                             */
/***********************************************************************/

public:
  /**
   * Standard "subclass factory" design pattern
   */
  static AISearchAgent* MakeNewAgent( int type, OpeningBook* ref );

  AISearchAgent();
  virtual ~AISearchAgent();

  /**
   * Pick a function which the agent will use to assess the potency of a
   * position.  This may change during the game; for example, a special
   * "mop-up" evaluator may replace the standard when it comes time to drive
   * a decisive advantage home at the end of the game.
   */
  bool AttachEvaluator( BoardEvaluator* eval );
  /**
   * The basic alpha-beta algorithm, used in one disguise or another by
   * every search agent class
   */
  int AlphaBeta( bool nodeType, Board* theBoard, int depth,
                        int alpha, int beta );
  int BasicAlphaBeta(bool nodeType, Board *theBoard, int depth, int alpha, int beta, bool sort = false);
  int BasicAlphaBetaTT( bool nodeType, Board* theBoard, int depth, int alpha, int beta, bool sort = false);
  int BasicAlphaBetaTTNew(bool nodeType, Board *theBoard, int depth, int alpha, int beta, bool sort = false);

  virtual istream& read( istream& is );

  int MiniMax(bool nodeType, Board *theBoard, int depth);
  int MT(bool nodeType, Board *theBoard, int depth, int esti);
  /**
   * Each agent class needs some way of picking a move!
   */
  virtual void PickBestMove( Board* theBoard, Move* bestMove );
  /**
   * A slight variant of alphabeta which only considers captures and null moves
   * This is necesary because the evaluation function can only be applied to
   * "quiet" positions where the tactical situation ( i.e., material balance ) is
   * unlikely to change in the near future.
   * Note that, in this version of the code, the quiescence search is not limited
   * by depth; we continue digging for as long as we can find captures.  Some other
   * programs impose a depth limit for time-management purposes.
   */
  int QuiescenceSearch( bool nodeType, Board* theBoard, int alpha, int beta );

  virtual void ShowStats();

	void TestAgent(Board* theBoard);

  virtual ostream& write( ostream& os ) const;
};

#endif // !defined( AFX_AISEARCHAGENT_H__B1DEBB74_4ACA_4FC8_ADEB_CDA8861DED4B__INCLUDED_ )
