// BoardEvaluator.cpp: Implementierung der Klasse BoardEvaluator.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BoardEvaluator.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


/***************************************************************************
 * Static Class members
 **************************************************************************/

const int BoardEvaluator::GRAIN	= 3;
const int BoardEvaluator::EVALUATE_BADBISHOPS		= 0;
const int BoardEvaluator::EVALUATE_BADKNIGHTS		= 1;
const int BoardEvaluator::EVALUATE_COMPLETE			= 2;
const int BoardEvaluator::EVALUATE_DEVELOPMENT		= 3;
const int BoardEvaluator::EVALUATE_KINGTROPISM		= 4;
const int BoardEvaluator::EVALUATE_MATERIAL			= 5;
const int BoardEvaluator::EVALUATE_PAWNSTRUCTURE	= 6;
const int BoardEvaluator::EVALUATE_ROOKBONUS		= 7;
const int BoardEvaluator::EVALUATE_ARRAY_SIZE		= 8;
 
/***************************************************************************
 * Constructor / Destructor
 **************************************************************************/
BoardEvaluator::BoardEvaluator()
{
	int i;

	PawnRams = 0;
	MaxTotalPawns = 0;

	for( i = 0; i < 2; i++ ) {
		MaxPawnColorBins[ i ] = 0;
	}
	for( i = 0; i < 8; i++ ) {
		MaxPawnFileBins[ i ] = 0;
		MaxMostAdvanced[ i ] = 0;
		MaxPassedPawns [ i ] = 0;
		MinPawnFileBins[ i ] = 0;
		MinMostBackward[ i ] = 0;
	}
}



BoardEvaluator::~BoardEvaluator()
{

}

/***************************************************************************
 * Operations
 **************************************************************************/
int BoardEvaluator::EvaluateQuickie( Board *theBoard, int fromWhosePerspective, bool useGrain )
{
	if ( useGrain ) {
		return ( ( theBoard->EvalMaterial( fromWhosePerspective ) >> GRAIN ) << GRAIN );
	} else {
		return theBoard->EvalMaterial( fromWhosePerspective );
	}
}



int BoardEvaluator::EvaluateComplete( Board *theBoard, int fromWhosePerspective, bool useGrain )
{
    AnalyzePawnStructure( theBoard, fromWhosePerspective );
	if ( useGrain ) {
		return( ( ( theBoard->EvalMaterial( fromWhosePerspective ) + 
				 EvalPawnStructure( fromWhosePerspective ) + 
				 EvalBadBishops( theBoard, fromWhosePerspective ) + 
				 EvalBadKnights( theBoard, fromWhosePerspective ) + 
				 EvalDevelopment( theBoard, fromWhosePerspective ) + 
				 EvalRookBonus( theBoard, fromWhosePerspective ) + 
				 EvalKingTropism( theBoard, fromWhosePerspective ) ) >> GRAIN ) << GRAIN );
	} else {
		return ( theBoard->EvalMaterial( fromWhosePerspective ) + 
				 EvalPawnStructure( fromWhosePerspective ) + 
				 EvalBadBishops( theBoard, fromWhosePerspective ) + 
				 EvalBadKnights( theBoard, fromWhosePerspective ) + 
				 EvalDevelopment( theBoard, fromWhosePerspective ) + 
				 EvalRookBonus( theBoard, fromWhosePerspective ) + 
				 EvalKingTropism( theBoard, fromWhosePerspective ) );
	}
}




int BoardEvaluator::EvalKingTropism( Board *theBoard, int fromWhosePerspective )
{
	int score = 0;

	// Square coordinates
	int kingRank = 0, kingFile = 0;
	int pieceRank = 0, pieceFile = 0;
	int i;

	if ( fromWhosePerspective == Player::SIDE_WHITE ) {
		// Look for enemy king first!
		for( i = 0; i < 64; i++ ) {
			if ( theBoard->FindBlackPiece( i ) == Board::BLACK_KING ) {
				kingRank = i >> 8;
				kingFile = i % 8;
				break;
			}
		}

		// Now, look for pieces which need to be evaluated
		for( i = 0; i < 64; i++ ) {
			pieceRank = i >> 8;
			pieceFile = i % 8;
			if ( theBoard->FindWhitePiece( i ) == Board::WHITE_ROOK ) {
				score -= ( min( abs( kingRank - pieceRank ), abs( kingFile - pieceFile ) ) << 1 );
			} else if ( theBoard->FindWhitePiece( i ) == Board::WHITE_KNIGHT ) {
				score += 5 - abs( kingRank - pieceRank ) - abs( kingFile - pieceFile );
			} else if ( theBoard->FindWhitePiece( i ) == Board::WHITE_QUEEN ) {
				score -= min( abs( kingRank - pieceRank ), abs( kingFile - pieceFile ) );
			}
		}
	} else { 
		// fromWhosePerspective == Player::SIDE_BLACK
		// Look for enemy king first!
		for( i = 63; i >= 0; i-- ) {
			if ( theBoard->FindWhitePiece( i ) == Board::WHITE_KING ) {
				kingRank = i >> 8;
				kingFile = i % 8;
				break;
			}
		}

		// Now, look for pieces which need to be evaluated
		for( i = 0; i < 64; i++ ) {
			pieceRank = i >> 8;
			pieceFile = i % 8;
			if ( theBoard->FindBlackPiece( i ) == Board::BLACK_ROOK ) {
				score -= ( min( abs( kingRank - pieceRank ), abs( kingFile - pieceFile ) ) << 1 );
			} else if ( theBoard->FindBlackPiece( i ) == Board::BLACK_KNIGHT ) {
				score += 5 - abs( kingRank - pieceRank ) - abs( kingFile - pieceFile );
			} else if ( theBoard->FindBlackPiece( i ) == Board::BLACK_QUEEN ) {
				score -= min( abs( kingRank - pieceRank ), abs( kingFile - pieceFile ) );
			}	
		}
	}
	return score;
}



int BoardEvaluator::EvalRookBonus( Board *theBoard, int fromWhosePerspective )
{
	__int64 rookboard = theBoard->GetBitBoard( Board::ROOK + fromWhosePerspective );
  
	if ( rookboard == 0 )
		return 0;

	int score = 0;
  
	for( int square = 0; square < 64; square++ ) {
		// Find a rook
		if ( ( rookboard & Board::SquareBits[ square ] ) != 0 ) {
			// Is this rook on the seventh rank?
			int rank = ( square >> 3 );
			int file = ( square % 8 );
			if ( ( fromWhosePerspective == Player::SIDE_WHITE ) && ( rank == 1 ) )
				score += 22;
			if ( ( fromWhosePerspective == Player::SIDE_BLACK ) && ( rank == 7 ) )
				score += 22;

			// Is this rook on a semi- or completely open file?
			if ( MaxPawnFileBins[ file ] == 0 ) {
				if ( MinPawnFileBins[ file ] == 0 )
					score += 10;
					else
					score += 4;
			}

			// Is this rook behind a passed pawn?
			if ( ( fromWhosePerspective == Player::SIDE_WHITE ) && ( MaxPassedPawns[ file ] < square ) )
				score += 25;
			if ( ( fromWhosePerspective == Player::SIDE_BLACK ) && ( MaxPassedPawns[ file ] > square ) )
				score += 25;

			// Use the bitboard erasure trick to avoid looking for additional
			// rooks once they have all been seen
			rookboard ^= Board::SquareBits[ square ];
			if ( rookboard == 0i64 )
				break;
		}
    }
	return score;
}



int BoardEvaluator::EvalDevelopment( Board *theBoard, int fromWhosePerspective )
{
	int score = 0;

	if ( fromWhosePerspective == Player::SIDE_WHITE ) {
		// Has the machine advanced its center pawns?
		if ( theBoard->FindWhitePiece( 51 ) == Board::WHITE_PAWN )
			score -= 15;
		if ( theBoard->FindWhitePiece( 52 ) == Board::WHITE_PAWN )
			score -= 15;

		// Penalize bishops and knights on the back rank
		for( int square = 56; square < 64; square++ ) {
			if ( ( theBoard->FindWhitePiece( square ) == Board::WHITE_KNIGHT ) ||
				 ( theBoard->FindWhitePiece( square ) == Board::WHITE_BISHOP ) )
				score -= 10;
		}

		// Penalize too-early queen movement
		__int64 queenboard = theBoard->GetBitBoard( Board::WHITE_QUEEN );
		if ( ( queenboard != 0 ) && ( ( queenboard & Board::SquareBits[ 59 ] ) == 0i64 ) ) {
			// First, count friendly pieces on their original squares
			int cnt = 0;
			if ( ( theBoard->GetBitBoard( Board::WHITE_BISHOP ) & Board::SquareBits[ 58 ] ) != 0i64 )
				cnt++;
			if ( ( theBoard->GetBitBoard( Board::WHITE_BISHOP ) & Board::SquareBits[ 61 ] ) != 0i64 )
				cnt++;
			if ( ( theBoard->GetBitBoard( Board::WHITE_KNIGHT ) & Board::SquareBits[ 57 ] ) != 0i64 )
				cnt++;
			if ( ( theBoard->GetBitBoard( Board::WHITE_KNIGHT ) & Board::SquareBits[ 62 ] ) != 0i64 )
				cnt++;
			if ( ( theBoard->GetBitBoard( Board::WHITE_ROOK )	& Board::SquareBits[ 56 ] ) != 0i64 )
				cnt++;
			if ( ( theBoard->GetBitBoard( Board::WHITE_ROOK )	& Board::SquareBits[ 63 ] ) != 0i64 )
				cnt++;
			if ( ( theBoard->GetBitBoard( Board::WHITE_KING )	& Board::SquareBits[ 60 ] ) != 0i64 )
				cnt++;
			score -= ( cnt << 3 );
		}

		// And finally, incite castling when the enemy has a queen on the board
		// This is a slightly simpler version of a factor used by Cray Blitz
		if ( theBoard->GetBitBoard( Board::BLACK_QUEEN ) != 0i64 ) {
			// Being castled deserves a bonus
			if ( theBoard->GetHasCastled( Player::SIDE_WHITE ) )
				score += 10;
				// small penalty if you can still castle on both sides
			else if ( theBoard->GetCastlingStatus( Player::SIDE_WHITE + Board::CASTLE_QUEENSIDE ) &&
					  theBoard->GetCastlingStatus( Player::SIDE_WHITE + Board::CASTLE_QUEENSIDE ) )
				score -= 24;
			// bigger penalty if you can only castle kingside
			else if ( theBoard->GetCastlingStatus( Player::SIDE_WHITE + Board::CASTLE_KINGSIDE ) )
				score -= 40;
			// bigger penalty if you can only castle queenside
			else if ( theBoard->GetCastlingStatus( Player::SIDE_WHITE + Board::CASTLE_QUEENSIDE ) )
				score -= 80;
			// biggest penalty if you can't castle at all
			else
				score -= 120;
		}
    } else { // fromWhosePerspective != Player::SIDE_WHITE
		// Has the machine advanced its center pawns?
		if ( theBoard->FindBlackPiece( 11 ) == Board::BLACK_PAWN )
			score -= 15;
		if ( theBoard->FindBlackPiece( 12 ) == Board::BLACK_PAWN )
			score -= 15;

		// Penalize bishops and knights on the back rank
		for( int square = 0; square < 8; square++ ) {
			if ( ( theBoard->FindBlackPiece( square ) == Board::BLACK_KNIGHT ) ||
				 ( theBoard->FindBlackPiece( square ) == Board::BLACK_BISHOP ) )
				score -= 10;
		}

		// Penalize too-early queen movement
		__int64 queenboard = theBoard->GetBitBoard( Board::BLACK_QUEEN );
		if ( ( queenboard != 0 ) && ( ( queenboard & Board::SquareBits[ 3 ] ) == 0i64 ) ) {
			// First, count friendly pieces on their original squares
			int cnt = 0;
			if ( ( theBoard->GetBitBoard( Board::BLACK_BISHOP ) & Board::SquareBits[ 2 ] ) != 0i64 )
				cnt++;
			if ( ( theBoard->GetBitBoard( Board::BLACK_BISHOP ) & Board::SquareBits[ 5 ] ) != 0i64 )
				cnt++;
			if ( ( theBoard->GetBitBoard( Board::BLACK_KNIGHT ) & Board::SquareBits[ 1 ] ) != 0i64 )
				cnt++;
			if ( ( theBoard->GetBitBoard( Board::BLACK_KNIGHT ) & Board::SquareBits[ 6 ] ) != 0i64 )
				cnt++;
			if ( ( theBoard->GetBitBoard( Board::BLACK_ROOK )	& Board::SquareBits[ 0 ] ) != 0i64 )
				cnt++;
			if ( ( theBoard->GetBitBoard( Board::BLACK_ROOK )	& Board::SquareBits[ 7 ] ) != 0i64 )
				cnt++;
			if ( ( theBoard->GetBitBoard( Board::BLACK_KING )	& Board::SquareBits[ 4 ] ) != 0i64 )
				cnt++;
			score -= ( cnt << 3 );
		}

		// And finally, incite castling when the enemy has a queen on the board
		// This is a slightly simpler version of a factor used by Cray Blitz
		if ( theBoard->GetBitBoard( Board::WHITE_QUEEN ) != 0i64 ) {
			// Being castled deserves a bonus
			if ( theBoard->GetHasCastled( Player::SIDE_BLACK ) )
				score += 10;
			// small penalty if you can still castle on both sides
			else if ( theBoard->GetCastlingStatus( Player::SIDE_BLACK + Board::CASTLE_QUEENSIDE ) &&
					  theBoard->GetCastlingStatus( Player::SIDE_BLACK + Board::CASTLE_QUEENSIDE ) )
				score -= 24;
			// bigger penalty if you can only castle kingside
			else if ( theBoard->GetCastlingStatus( Player::SIDE_BLACK + Board::CASTLE_KINGSIDE ) )
				score -= 40;
			// bigger penalty if you can only castle queenside
			else if ( theBoard->GetCastlingStatus( Player::SIDE_BLACK + Board::CASTLE_QUEENSIDE ) )
				score -= 80;
			// biggest penalty if you can't castle at all
			else
				score -= 120;
		}
    }
	return score;
}



int BoardEvaluator::EvalBadBishops( Board *theBoard, int fromWhosePerspective )
{
	__int64 where = theBoard->GetBitBoard( Board::BISHOP + fromWhosePerspective );
	if ( where == 0 )
		return 0;

	int score = 0;
	for( int square = 0; square < 64; square++ ) {
		// Find a bishop
		if ( ( where & Board::SquareBits[ square ] ) != 0i64 ) {
			// What is the bishop's square color?
			int rank = ( square >> 3 );
			int file = ( square % 8 );
			if ( ( rank % 2 ) == ( file % 2 ) )
				score -= ( MaxPawnColorBins[ 0 ] << 3 );
				else
				score -= ( MaxPawnColorBins[ 1 ] << 3 );

			// Use the bitboard erasure trick to avoid looking for additional
			// bishops once they have all been seen
			where ^= Board::SquareBits[ square ];
			if ( where == 0i64 )
				break;
		}
    }
	return score;
}



int BoardEvaluator::EvalBadKnights( Board *theBoard, int fromWhosePerspective )
{
	__int64 where = theBoard->GetBitBoard( Board::KNIGHT + fromWhosePerspective );
	short   numKnights = 2;
	if ( where == 0 )
		return 0;

	int score = 0;
	int square = 0;
	
	// look for knights in UPPER RANK
	for( square = 0; square < 8; square++ ) {
		// Find a knight
		if ( ( where & Board::SquareBits[ square ] ) != 0i64 ) {
			score -= 10;

			// Avoid looking for additional knights, once
			// we have found both
			if ( --numKnights == 0 )
				return score;
		}
    }
	// look for knights in BACK RANK
	// do nothing, because knight at the back rank is
	// penalized in "EvalDevelopment()"
	// look for knights in UPPER RANK
	for( square = 56; square < 64; square++ ) {
		// Find a knight
		if ( ( where & Board::SquareBits[ square ] ) != 0i64 ) {
			score -= 10;

			// Avoid looking for additional knights, once
			// we have found both
			if ( --numKnights == 0 )
				return score;
		}
    }

	// look for knights in 1st FILE
	for( square = 8; square < 56; square += 8 ) {
		// Find a knight
		if ( ( where & Board::SquareBits[ square ] ) != 0i64 ) {
			score -= 20;

			// Avoid looking for additional knights, once
			// we have found both
			if ( --numKnights == 0 )
				return score;
		}
    }

	// look for knights in 8th FILE
	for( square = 15; square < 63; square += 8 ) {
		// Find a knight
		if ( ( where & Board::SquareBits[ square ] ) != 0i64 ) {
			score -= 20;

			// Avoid looking for additional knights, once
			// we have found both
		if ( --numKnights == 0 )
			return score;
		}
    }
	return score;
}



int BoardEvaluator::EvalPawnStructure( int fromWhosePerspective )
{
	int score = 0;
	int bin;

	// First, look for doubled pawns
	// In chess, two or more pawns on the same file usually hinder each other,
	// so we assign a minor penalty
	for( bin = 0; bin < 8; bin++ )
		if ( MaxPawnFileBins[ bin ] > 1 )
			score -= 8;

	// Now, look for an isolated pawn, i.e., one which has no neighbor pawns
	// capable of protecting it from attack at some point in the future
	if ( ( MaxPawnFileBins[ 0 ] > 0 ) && ( MaxPawnFileBins[ 1 ] == 0 ) )
		score -= 15;
	if ( ( MaxPawnFileBins[ 7 ] > 0 ) && ( MaxPawnFileBins[ 6 ] == 0 ) )
		score -= 15;
	for( bin = 1; bin < 7; bin++ ) {
		if ( ( MaxPawnFileBins[ bin ] > 0 ) && ( MaxPawnFileBins[ bin - 1 ] == 0 )
          && ( MaxPawnFileBins[ bin + 1 ] == 0 ) )
			score -= 15;
    }

	// Assign a small penalty to positions in which Max still has all of his
	// pawns; this incites a single pawn trade ( to open a file ), but not by
	// much
	if ( MaxTotalPawns == 8 )
		score -= 10;

	// Penalize pawn rams, because they restrict movement
	score -= 8 * PawnRams;

	// Finally, look for a passed pawn; i.e., a pawn which can no longer be
	// blocked or attacked by a rival pawn
	if ( fromWhosePerspective == Player::SIDE_WHITE ) {
		if ( MaxMostAdvanced[ 0 ] < min( MinMostBackward[ 0 ], MinMostBackward[ 1 ] ) )
			score += ( 8 - ( MaxMostAdvanced[ 0 ] >> 3 ) ) *
                     ( 8 - ( MaxMostAdvanced[ 0 ] >> 3 ) );
		if ( MaxMostAdvanced[ 7 ] < min( MinMostBackward[ 7 ], MinMostBackward[ 6 ] ) )
			score += ( 8 - ( MaxMostAdvanced[ 7 ] >> 3 ) ) *
					 ( 8 - ( MaxMostAdvanced[ 7 ] >> 3 ) );
		for( int i = 1; i < 7; i++ ) {
			if ( ( MaxMostAdvanced[ i ] < MinMostBackward[ i ] ) &&
				 ( MaxMostAdvanced[ i ] < MinMostBackward[ i - 1 ] ) &&
				 ( MaxMostAdvanced[ i ] < MinMostBackward[ i + 1 ] ) )
				score += ( 8 - ( MaxMostAdvanced[ i ] >> 3 ) ) *
                         ( 8 - ( MaxMostAdvanced[ i ] >> 3 ) );
		}
    } else { // from Black's perspective
		if ( MaxMostAdvanced[ 0 ] > max( MinMostBackward[ 0 ], MinMostBackward[ 1 ] ) )
			score += ( MaxMostAdvanced[ 0 ] >> 3 ) *
				     ( MaxMostAdvanced[ 0 ] >> 3 );
		if ( MaxMostAdvanced[ 7 ] > max( MinMostBackward[ 7 ], MinMostBackward[ 6 ] ) )
			score += ( MaxMostAdvanced[ 7 ] >> 3 ) *
				     ( MaxMostAdvanced[ 7 ] >> 3 );
		for( int i = 1; i < 7; i++ ) {
			if ( ( MaxMostAdvanced[ i ] > MinMostBackward[ i ] ) &&
				 ( MaxMostAdvanced[ i ] > MinMostBackward[ i - 1 ] ) &&
				 ( MaxMostAdvanced[ i ] > MinMostBackward[ i + 1 ] ) )
				score += ( MaxMostAdvanced[ i ] >> 3 ) *
					     ( MaxMostAdvanced[ i ] >> 3 );
		}
    }

	return score;
}



/**
  * Look at pawn positions to be able to detect features such as doubled,
  * isolated or passed pawns
  */
bool BoardEvaluator::AnalyzePawnStructure( Board *theBoard, int fromWhosePerspective ) {
	int i;
  
	// Reset the counters
	for( i = 0; i < 8; i++ ) {
		MaxPawnFileBins[ i ] = 0;
		MinPawnFileBins[ i ] = 0;
    }
	MaxPawnColorBins[ 0 ] = 0;
	MaxPawnColorBins[ 1 ] = 0;
	PawnRams = 0;
	MaxTotalPawns = 0;

	// Now, perform the analysis
	if ( fromWhosePerspective == Player::SIDE_WHITE ) {
		for( i = 0; i < 8; i++ ) {
			MaxMostAdvanced[ i ] = 63;
			MinMostBackward[ i ] = 63;
			MaxPassedPawns [ i ] = 63;
		}
		for( int square = 55; square >= 8; square-- ) {
			// Look for a white pawn first, and count its properties
			if ( theBoard->FindWhitePiece( square ) == Board::WHITE_PAWN ) {
				// What is the pawn's position, in rank-file terms?
				int rank = square >> 3;
				int file = square % 8;

				// This pawn is now the most advanced of all white pawns on its file
				MaxPawnFileBins[ file ]++;
				MaxTotalPawns++;
				MaxMostAdvanced[ file ] = square;

				// Is this pawn on a white or a black square?
				if ( ( rank % 2 ) == ( file % 2 ) )
					MaxPawnColorBins[ 0 ]++;
					else
					MaxPawnColorBins[ 1 ]++;

				// Look for a "pawn ram", i.e., a situation where a black pawn
				// is located in the square immediately ahead of this one.
				if ( theBoard->FindBlackPiece( square - 8 ) == Board::BLACK_PAWN )
					PawnRams++;
			}
			// Now, look for a BLACK pawn
			else if ( theBoard->FindBlackPiece( square ) == Board::BLACK_PAWN ) {
				// If the black pawn exists, it is the most backward found so far
				// on its file
				int file = square % 8;
				MinPawnFileBins[ file ]++;
				MinMostBackward[ file ] = square;
			}
		}
    } else { // Analyze from Black's perspective
		for( int i = 0; i < 8; i++ ) {
			MaxMostAdvanced[ i ] = 0;
			MaxPassedPawns [ i ] = 0;
			MinMostBackward[ i ] = 0;
		}
		for( int square = 8; square < 56; square++ ) {
			if ( theBoard->FindBlackPiece( square ) == Board::BLACK_PAWN ) {
				// What is the pawn's position, in rank-file terms?
				int rank = square >> 3;
				int file = square % 8;

				// This pawn is now the most advanced of all white pawns on its file
				MaxPawnFileBins[ file ]++;
				MaxTotalPawns++;
				MaxMostAdvanced[ file ] = square;

				if ( ( rank % 2 ) == ( file % 2 ) )
					MaxPawnColorBins[ 0 ]++;
					else
					MaxPawnColorBins[ 1 ]++;

				if ( theBoard->FindWhitePiece( square + 8 ) == Board::WHITE_PAWN )
					PawnRams++;
			} else if ( theBoard->FindWhitePiece( square ) == Board::WHITE_PAWN ) {
				int file = square % 8;
				MinPawnFileBins[ file ]++;
				MinMostBackward[ file ] = square;
			}
		}
    }
	return true;
}


void BoardEvaluator::PrintVerbose( Board *theBoard, int fromWhosePerspective, bool newBoard )
{

    *Output::cout << "BoardEvaluator - side: ";
	if ( fromWhosePerspective == Player::SIDE_WHITE )
		*Output::cout << "WHITE";
		else
		*Output::cout << "BLACK"; 
	*Output::cout << endl;

	int Evals[ 8 ];
	int *diffEvals;

	AnalyzePawnStructure( theBoard, fromWhosePerspective );
	Evals[ EVALUATE_MATERIAL ]		= theBoard->EvalMaterial( fromWhosePerspective );
	Evals[ EVALUATE_PAWNSTRUCTURE ]	= EvalPawnStructure( fromWhosePerspective );
	Evals[ EVALUATE_BADBISHOPS ]	= EvalBadBishops ( theBoard, fromWhosePerspective );
	Evals[ EVALUATE_BADKNIGHTS ]	= EvalBadKnights ( theBoard, fromWhosePerspective );
	Evals[ EVALUATE_DEVELOPMENT ]	= EvalDevelopment( theBoard, fromWhosePerspective );
	Evals[ EVALUATE_ROOKBONUS ]		= EvalRookBonus  ( theBoard, fromWhosePerspective );
	Evals[ EVALUATE_KINGTROPISM ]	= EvalKingTropism( theBoard, fromWhosePerspective );
	int i;
    for ( Evals[ EVALUATE_COMPLETE ] = i = 0; i < EVALUATE_ARRAY_SIZE; i ++ )
		if ( i != EVALUATE_COMPLETE ) Evals[ EVALUATE_COMPLETE ] += Evals[ i ];

	diffEvals = BoardEvaluatorComparator::getDiff( Evals, fromWhosePerspective );

	*Output::cout << setiosflags( ios::showpos );
    *Output::cout << " - EvalMaterial\t\t" 
					<< setw( 5 ) << Evals[ EVALUATE_MATERIAL ] 
					<< "  ( d = " << setw( 4 ) << diffEvals[ EVALUATE_MATERIAL ] << " )" << endl;
    *Output::cout << " - EvalPawnStructure\t" 
					<< setw( 5 ) << Evals[ EVALUATE_PAWNSTRUCTURE ] 
					<< "  ( d = " << setw( 4 ) << diffEvals[ EVALUATE_PAWNSTRUCTURE ] << " )" << endl;
    *Output::cout << " - EvalBadBishops\t" 
					<< setw( 5 ) << Evals[ EVALUATE_BADBISHOPS ] 
					<< "  ( d = " << setw( 4 ) << diffEvals[ EVALUATE_BADBISHOPS ] << " )" << endl;
    *Output::cout << " - EvalBadKnights\t" 
					<< setw( 5 ) << Evals[ EVALUATE_BADKNIGHTS ] 
					<< "  ( d = " << setw( 4 ) << diffEvals[ EVALUATE_BADKNIGHTS ] << " )" << endl;
    *Output::cout << " - EvalDevelopment\t" 
					<< setw( 5 ) << Evals[ EVALUATE_DEVELOPMENT ] 
					<< "  ( d = " << setw( 4 ) << diffEvals[ EVALUATE_DEVELOPMENT ] << " )" << endl;
    *Output::cout << " - EvalRookBonus\t" 
					<< setw( 5 ) << Evals[ EVALUATE_ROOKBONUS ] 
					<< "  ( d = " << setw( 4 ) << diffEvals[ EVALUATE_ROOKBONUS ] << " )" << endl;
    *Output::cout << " - EvalKingTropism\t" 
					<< setw( 5 ) << Evals[ EVALUATE_KINGTROPISM ] 
					<< "  ( d = " << setw( 4 ) << diffEvals[ EVALUATE_KINGTROPISM ] << " )" << endl;
	*Output::cout << " > Complete\t\t" 
					<< setw( 5 ) << ( ( Evals[ EVALUATE_COMPLETE ] >> GRAIN ) << GRAIN ) 
					<< "  ( d = " << setw( 4 ) << ( ( ( Evals[ EVALUATE_COMPLETE ] >> GRAIN ) << GRAIN ) - ( ( ( Evals[ EVALUATE_COMPLETE ] - diffEvals[ EVALUATE_COMPLETE ] ) >> GRAIN ) << GRAIN ) ) << " )" << endl;
    *Output::cout << " > without grain\t" 
					<< setw( 5 ) << Evals[ EVALUATE_COMPLETE ] 
					<< "  ( d = " << setw( 4 ) << diffEvals[ EVALUATE_COMPLETE ] << " )" << endl;

	*Output::cout << resetiosflags( ios::showpos );

	if ( newBoard )
		BoardEvaluatorComparator::NewBoard( Evals, fromWhosePerspective );
	
}
