// Move.cpp: Implementierung der Klasse Move.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Move.h"



/***************************************************************************
 * Static Class members
 **************************************************************************/
const int Move::MOVE_NORMAL				      =   0;
const int Move::MOVE_CAPTURE_ORDINARY	  =   1;
const int Move::MOVE_CAPTURE_EN_PASSANT	=   2;
const int Move::MOVE_CASTLING_KINGSIDE	=   4;
const int Move::MOVE_CASTLING_QUEENSIDE =   8;

const int Move::MOVE_RESIGN				    =  16;
const int Move::MOVE_STALEMATE		    =  17;     
const int Move::MOVE_CHECKMATE        =  18;     
const int Move::MOVE_PROMOTION_KNIGHT	=  32;
const int Move::MOVE_PROMOTION_BISHOP	=  64;
const int Move::MOVE_PROMOTION_ROOK		= 128;
const int Move::MOVE_PROMOTION_QUEEN	= 256;
const int Move::PROMOTION_MASK			  = 480;
const int Move::NO_PROMOTION_MASK		  =  31;
const int Move::EVALTYPE_ACCURATE		  =   0;
const int Move::EVALTYPE_UPPERBOUND		=   1;
const int Move::EVALTYPE_LOWERBOUND		=   2;
const int Move::NULL_MOVE				      =  -1;

const int Move::MOVE_RATING_UNKNOWN		=   0;
const int Move::MOVE_RATING_GOOD		  =   1;
const int Move::MOVE_RATING_POOR	    =   2;

/***************************************************************************
 * Constructor / Destructor
 **************************************************************************/

Move::Move()
{
	Reset();
}

Move::~Move()
{

}
/***************************************************************************
 * Operations
 **************************************************************************/

bool Move::Set( Move *target )
{
  if ( target == NULL ) return false;
  memcpy( this, target, sizeof( Move ) );
/*
  CapturedPiece = target->CapturedPiece;
  DestinationSquare = target->DestinationSquare;
  MoveEvaluation = target->MoveEvaluation;
  MoveEvaluationType = target->MoveEvaluationType;
  MoveType = target->MoveType;
  MovingPiece = target->MovingPiece;
  SearchDepth = target->SearchDepth;
  SourceSquare = target->SourceSquare;
*/
  return true;
}




bool Move::Equals( Move *target )
{
	if ( target == NULL ) return false;

	if ( MovingPiece != target->MovingPiece )
		return false;
	if ( CapturedPiece != target->CapturedPiece )
		return false;
	if ( MoveType != target->MoveType )
		return false;
	if ( SourceSquare != target->SourceSquare )
		return false;
	if ( DestinationSquare != target->DestinationSquare )
		return false;
	return true;
}



bool Move::Reset()
{
    MovingPiece = Board::EMPTY_SQUARE;
    CapturedPiece = Board::EMPTY_SQUARE;
    SourceSquare = NULL_MOVE;
    DestinationSquare = NULL_MOVE;
    MoveType = NULL_MOVE;
    MoveEvaluation = NULL_MOVE;
    MoveEvaluationType = NULL_MOVE;
    SearchDepth = NULL_MOVE;
    return true;
}



void Move::Print()
{
    if ( MoveType == MOVE_STALEMATE ) {
		*Output::cout <<  "STALEMATE!!!" << endl;
	}
    if ( MoveType != MOVE_RESIGN ) {
		*Output::cout <<  Board::PieceStrings[ MovingPiece ];
		string movestr;
		Board::SquareToCoord( SourceSquare, DestinationSquare, &movestr );
		*Output::cout <<  " - " << movestr << endl;
	} else {
		*Output::cout <<  "RESIGNATION!" << endl;
	}
}


void Move::ThinkPrint()
{
    if ( MoveType == MOVE_STALEMATE ) {
		*Output::cout <<  " STALE" << endl;
	}
    if ( MoveType != MOVE_RESIGN ) {
		string movestr;
		Board::SquareToCoord( SourceSquare, DestinationSquare, &movestr );
		*Output::cout <<  " " << movestr;
	} else {
		*Output::cout <<  " RESIGN" << endl;
	}
}



bool Move::PrintToSAN( string *s )
{
	if ( s == NULL ) return false;
	
	// if one player resigned
	if ( MoveType == MOVE_RESIGN ) {
		*s = "";
		return true;
	}

	string tempStr( "" );
	
	// The first letter represents the moving piece
	// see "8.2.3.2: Piece identification" of the PGN Spec
	if ( MovingPiece == Board::BISHOP )
		*s = "B";
	else if ( MovingPiece == Board::KING )
		*s = "K";
	else if ( MovingPiece == Board::QUEEN )
		*s = "Q";
	else if ( MovingPiece == Board::ROOK )
		*s = "R";
	else if ( MovingPiece == Board::KNIGHT )
		*s = "N"; 
	else if ( MovingPiece == Board::PAWN )
		*s = "";

	// .. to avoid ambiguity here comes the sourcequare,
	// see 8.2.3.4: Disambiguation
	Board::SquareToCoord( SourceSquare, &tempStr );
	*s += tempStr;

	// see 8.2.3.3: Basic SAN move construction
	if ( MoveType == MOVE_CAPTURE_EN_PASSANT || MoveType == MOVE_CAPTURE_ORDINARY )
		*s += "x";
	else if ( MoveType == MOVE_CASTLING_QUEENSIDE )	{
		*s += " O-O";
		return true;
	} else if ( MoveType == MOVE_CASTLING_QUEENSIDE ) {
		*s += " O-O-O";
		return true;
	} else if ( MoveType == MOVE_PROMOTION_BISHOP )
		*s += "=B";
	else if ( MoveType == MOVE_PROMOTION_KNIGHT )
		*s += "=N";
	else if ( MoveType == MOVE_PROMOTION_QUEEN )
		*s += "=Q";
	else if ( MoveType == MOVE_PROMOTION_ROOK )
		*s += "=R";

	
	// ... the next hyphens the move itself
	// see 8.2.3.1: Square identification
	Board::SquareToCoord( DestinationSquare, &tempStr );
	*s += tempStr;

	// ... checking or checkmate status
	// see 8.2.3.5: Check and checkmate indication characters
	
	// falls in Schach gesetzt: *movstr += " + "
	// falls Schachmatt gesetzt: *movstr += "#"
	return true;
}



bool Move::CheckIntegrity()
{
	static cnt = 0;
	bool   checkOk = true;
	cnt++;
	if ( ! ( MovingPiece >= 0 && MovingPiece <= Board::ALL_BITBOARDS ) 	 ) {
		*Output::cout << cnt << ": Integrity failed because of MovingPiece :" << MovingPiece << endl;
		checkOk = false;
	}
	if ( ! ( CapturedPiece >= 0 && CapturedPiece <= Board::ALL_BITBOARDS ) 	 ) {
		*Output::cout << cnt << ": Integrity failed because of CapturedPiece :" << CapturedPiece << endl;
		checkOk = false;
	}
	if ( ! ( SourceSquare == NULL_MOVE || ( SourceSquare >= 0 && SourceSquare <= 63 ) ) ) {
		*Output::cout << cnt << ": Integrity failed because of SourceSquare :" << SourceSquare << endl;
		checkOk = false;
	}
	if ( ! ( DestinationSquare == NULL_MOVE || ( DestinationSquare >= 0 && DestinationSquare <= 63 ) ) ) {
		*Output::cout << cnt << ": Integrity failed because of DestinationSquare :" << DestinationSquare << endl;
		checkOk = false;
	}
	if ( ! ( MoveType == NULL_MOVE || ( MoveType >= 0 && MoveType <= ( 512 << 1 ) ) ) ) {
		*Output::cout << cnt << ": Integrity failed because of MoveType :" << MoveType << endl;
		checkOk = false;
	}
	if ( ! ( MoveEvaluationType == NULL_MOVE || ( MoveEvaluationType >= EVALTYPE_ACCURATE && MoveEvaluationType <= EVALTYPE_LOWERBOUND ) ) ) {
		*Output::cout << cnt << ": Integrity failed because of MoveEvaluationType :" << MoveEvaluationType << endl;
		checkOk = false;
	}
	if ( ! ( SearchDepth >= NULL_MOVE ) ) {
		*Output::cout << cnt << ": Integrity failed because of SearchDepth :" << SearchDepth << endl;
		checkOk = false;
	}
	
	return checkOk;
}


ostream& Move::write( ostream &os ) const
{
	os.write( ( char* )&CapturedPiece,			sizeof( CapturedPiece ) );
	os.write( ( char* )&DestinationSquare,		sizeof( DestinationSquare ) );
	os.write( ( char* )&MoveEvaluation,			sizeof( MoveEvaluation ) );
	os.write( ( char* )&MoveEvaluationType,		sizeof( MoveEvaluationType ) );
	os.write( ( char* )&MoveType,				sizeof( MoveType ) );
	os.write( ( char* )&MovingPiece,			sizeof( MovingPiece ) );
	os.write( ( char* )&SearchDepth,			sizeof( SearchDepth ) );
	os.write( ( char* )&SourceSquare,			sizeof( SourceSquare ) );
	return os;
}

istream& Move::read( istream &is )
{
	is.read( ( char* )&CapturedPiece,			sizeof( CapturedPiece ) );
	is.read( ( char* )&DestinationSquare,		sizeof( DestinationSquare ) );
	is.read( ( char* )&MoveEvaluation,			sizeof( MoveEvaluation ) );
	is.read( ( char* )&MoveEvaluationType,		sizeof( MoveEvaluationType ) );
	is.read( ( char* )&MoveType,				sizeof( MoveType ) );
	is.read( ( char* )&MovingPiece,				sizeof( MovingPiece ) );
	is.read( ( char* )&SearchDepth,				sizeof( SearchDepth ) );
	is.read( ( char* )&SourceSquare,			sizeof( SourceSquare ) );
	return is;
}

/* compares to moves - used for stable_sort */
bool Move::operator<( const Move& other ) const{
	if ( CutoffHistory::pCurrentHistory[       SourceSquare * 64 +       DestinationSquare ] <
		 CutoffHistory::pCurrentHistory[ other.SourceSquare * 64 + other.DestinationSquare ] )
		return true;
		else
		return false;
}
