//
// OpeningBook - A hash table of well-known positions and moves
//
// Chess programs are notoriously bad at deciding what to do with complicated
// positions, so everyone "cheats" by giving them a library of opening positions
// taken from the ECO or something like that.  This one is very primitive and
// contains very little, but it gets the job done.
//
//
//////////////////////////////////////////////////////////////////////

#if !defined( AFX_OPENINGBOOK_H__C8820C64_EE1A_4F5F_A60C_A499B40C46B3__INCLUDED_ )
#define AFX_OPENINGBOOK_H__C8820C64_EE1A_4F5F_A60C_A499B40C46B3__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "OpeningBookEntry.h"

class OpeningBook  
{
/***************************************************************************
 * Static Constants
 **************************************************************************/
private:
	/** size of Hashtable OpeningBook.Table*/
  static const int TABLE_SIZE;
  static const __int8 FILETAG_NONE;
  static const __int8 FILETAG_EOF;
  static const __int8 FILETAG_NEXT;
  static const __int8 FILETAG_BLACK;

  int maxcollision;
  int entries;


/***************************************************************************
 * Data Members
 **************************************************************************/
private:
  /** The hash table itself */
  OpeningBookEntry **WhiteTable;
  OpeningBookEntry **BlackTable;
public:
  static bool used_for_last_move;
  static int variants;
  static int variantno;


	
/***************************************************************************
 * Constructor / Destructor
 **************************************************************************/
public:
	OpeningBook();
	OpeningBook( string& fileName );
	virtual ~OpeningBook();
	

/***************************************************************************
 * Operations
 **************************************************************************/
public:
	/**
		* Querying the table for a ready-made move to play.  Return null if there
	  * is none
	  */
	Move *Query( Board *theBoard );
	/**
	  * Loading the table from a file
		*/
	bool Load( string SourceFile );
	static bool StartBookCreation( int argc, char* argv[] );

private:
	bool Store( string destFile );
	int includePGN( string pgnFile );
	void AddMove( int theHash, int theLock, Move *openingMove, int rating, int currentPlayer );
	int max_creating_depth;

};

#endif // !defined( AFX_OPENINGBOOK_H__C8820C64_EE1A_4F5F_A60C_A499B40C46B3__INCLUDED_ )
