// PGNExporter.cpp: Implementierung der Klasse PGNExporter.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PGNExporter.h"

/***************************************************************************
 * Static Class members
 **************************************************************************/

const int PGNExporter::RESULT_UNKNOWN    = 0;
const int PGNExporter::RESULT_WHITE_WINS = 1;
const int PGNExporter::RESULT_BLACK_WINS = 2;
const int PGNExporter::RESULT_DRAW_GAME  = 3;


/***************************************************************************
 * Constructor / Destructor
 **************************************************************************/

PGNExporter::PGNExporter()
{
	time_t today;
	time( &today );
	
	SetGameEvent( "Casual Game" );
	SetGameDate( &today );
	SetGameRound( 0 );
	SetGameSite( "" );
	SetGameResult( RESULT_UNKNOWN );
	SetWhitePlayer ( "White Player" );
	SetBlackPlayer ( "Black Player" );

	FileName = "";

	CurrentPlayer = Player::SIDE_WHITE;
}




PGNExporter::~PGNExporter()
{

}

/***************************************************************************
 * Operations
 **************************************************************************/

bool PGNExporter::SetCurrentPlayer ( const int which )
{
	if ( which == Player::SIDE_BLACK || which == Player::SIDE_WHITE )
		{
		return false;
		}
	
	CurrentPlayer = which;
	return true;
}

bool PGNExporter::Create( char *filename )
{
	if ( ( FileHandler = fopen( filename , "w" ) ) == NULL )
		return false;

	FileName = filename;

	Moves.erase( Moves.begin(), Moves.end() );

	MoveCounter = 0;

	return true;
}


bool PGNExporter::Open( char *filename )
{
	if ( ( FileHandler = fopen( filename , "a" ) ) == NULL )
		return false;

	FileName = filename;

	Moves.erase( Moves.begin(), Moves.end() );

	MoveCounter = 0;

	return true;
}


bool PGNExporter::WriteTagPairs()
{
	string s ( *GameDate.toStringYYYYMMDD( '.' ) );
	
	if ( !FileHandler ) return false;

	if ( fprintf( FileHandler, "[Event \" % s\" ]\n",    GameEvent )   < 1 ) return false;
	if ( fprintf( FileHandler, "[Site \" % s\" ]\n",     GameSite )    < 1 ) return false;
	if ( fprintf( FileHandler, "[Date \" % s\" ]\n",     s.c_str() )   < 1 ) return false;
	if ( fprintf( FileHandler, "[Round \" % i\" ]\n",    GameRound )   < 1 ) return false;
	if ( fprintf( FileHandler, "[White \" % s\" ]\n",    PlayerWhite ) < 1 ) return false;
	if ( fprintf( FileHandler, "[Black \" % s\" ]\n",    PlayerBlack ) < 1 ) return false;
	if ( fprintf( FileHandler, "[Result \" % s\" ]\n\n", GameResult )  < 1 ) return false;
	
	MoveCounter++;

	return true;
}



bool PGNExporter::AppendMove( Move *mov )
{
	// Store move
	Moves.push_back( *mov );
	
	// Update MoveCounter and GameRound
	MoveCounter++;
	SetGameRound( MoveCounter / 2 );
	
	// a move contains Checkmate, Stalemate or Resign updates SRT
	if ( mov->MoveType == Move::MOVE_CHECKMATE )
			if ( CurrentPlayer == Player::SIDE_BLACK )
				SetGameResult( RESULT_BLACK_WINS );
			else
				SetGameResult( RESULT_WHITE_WINS );
	else if ( mov->MoveType == Move::MOVE_STALEMATE )
			SetGameResult( RESULT_DRAW_GAME );
	else if ( mov->MoveType == Move::MOVE_RESIGN )
			if ( CurrentPlayer == Player::SIDE_BLACK )
				SetGameResult( RESULT_WHITE_WINS );
			else
				SetGameResult( RESULT_BLACK_WINS );
			
	
	// Update Currentplayer
	if ( CurrentPlayer == Player::SIDE_BLACK )
		CurrentPlayer = Player::SIDE_WHITE;
	else
		CurrentPlayer = Player::SIDE_BLACK;
	
	return true;
}



bool PGNExporter::WriteMoves()
{
	MoveCounter = 1;

	for ( vector<Move>::iterator mov = Moves.begin(); mov != Moves.end(); mov++ ) {
		string s;
		mov->PrintToSAN( &s );
	
		if ( ( MoveCounter % 2 ) == 1 )
			fprintf( FileHandler, " % i. ", ( MoveCounter + 1 ) / 2 );
	
		fprintf( FileHandler, " % s ", s.c_str() );
	
		MoveCounter++;

		if ( MoveCounter % 11 == 0 ) {
			// overwrite whitespace; isn't allowed as last linecharacter
			fseek( FileHandler, -1, SEEK_CUR );
			fprintf( FileHandler, "\n" );
		}
	}

	return true;
}



bool PGNExporter::Close()
{
	if ( WriteTagPairs() != true )				{ fclose( FileHandler ); return NULL; }
	if ( WriteMoves() != true )					{ fclose( FileHandler ); return NULL; }
	if ( WriteGameTerminationMarker() != true ) { fclose( FileHandler ); return NULL; }
	
	fclose( FileHandler );

	return true;
}



bool PGNExporter::SetGameResult( int result )
{
	switch ( result ) {
		case RESULT_UNKNOWN :    strcpy( GameResult, "*" );			return true;
		case RESULT_BLACK_WINS : strcpy( GameResult, "0-1" );		return true;
		case RESULT_WHITE_WINS : strcpy( GameResult, "1-0" );		return true;
		case RESULT_DRAW_GAME :  strcpy( GameResult, "1/2-1/2" );	return true;
		default: return false;
	}

	return false;
}



bool PGNExporter::WriteGameTerminationMarker()
{
	return ( fprintf( FileHandler, " % s", GameResult ) > 0 );
}
