// PGNParser.cpp can parse a SAN formated chess file. Since SAN is a 
// really complicated and confusing format, the code to parse is it,
// too. 
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PGNParser.h"

/***************************************************************************
 * Static Class members
 **************************************************************************/

const int PGNParser::WINNER_WHITE   = 0;
const int PGNParser::WINNER_BLACK   = 1;
const int PGNParser::WINNER_DRAW    = 2;
const int PGNParser::WINNER_UNKNOWN = 3;

const int PGNParser::SIDE_WHITE     = Player::SIDE_WHITE;
const int PGNParser::SIDE_BLACK     = Player::SIDE_BLACK;

/***************************************************************************
 * Constructor / Destructor
 **************************************************************************/

PGNParser::PGNParser( istream& PGNFile )
{
	this->PGNFile = &PGNFile;
	matchNo = 0;
	winner = WINNER_UNKNOWN;
	PGNBoard.StartingBoard();
	brackets  = 0;
	brackets2 = 0;
	comment = false;
	takenext = false;
}

PGNParser::~PGNParser()
{

}

/***************************************************************************
 * Operations
 **************************************************************************/

bool PGNParser::nextMatch()
{
	PGNBoard.StartingBoard();
	PlayerWhite = "";
	PlayerBlack = "";
	winner = WINNER_UNKNOWN;
	matchNo++;
	moveNo = 0;
	char c;
	string header = "";
	bool skipoldgame = true;
	while ( !PGNFile->eof() ) {
 		PGNFile->read( &c, 1 );
		if ( c != EOF ) {
			if ( takenext ) {
				header += c;
				takenext = false;
				continue;
			}

			if ( c == '\\' ) {
				takenext = true;
				continue;
			}

			if ( c == '"' ) { comment = !comment; }

			if ( comment ) {
				if ( c == 13 || c == 10 ) {
					comment = false;
					brackets  = 0;
					brackets2 = 0;
					continue;
				}
				header += c;
				continue;
			}

			if ( c == '{' ) { brackets2++; continue;}
			if ( c == '}' ) { brackets2--; continue;}
			if ( c == '[' ) {
				skipoldgame = false;
				brackets++; 
				continue;
			}
			if ( c == ']' ) {
				brackets--; 
				if ( brackets == 0 ) {
					int pos;
					if ( ( ( pos = header.find( "Result" ) ) != string::npos ) || ( ( pos =  header.find( "result" ) ) != string::npos ) ) { // try to parse result of match
						header.erase( 0, 6 + pos ); // remove Result....
						if ( ( pos = header.find( "\"" ) ) != string::npos ) {
							header.erase( 0, 1 + pos );
							if ( ( pos = header.find( "\"" ) ) != string::npos ) {
								header = header.substr( 0, pos );
								if ( header == "*" )       winner = WINNER_UNKNOWN;
								if ( header == "1-0" )     winner = WINNER_WHITE;
								if ( header == "0-1" )     winner = WINNER_BLACK;
								if ( header == "1/2-1/2" ) winner = WINNER_DRAW;
							}
						}
					}
					if ( ( ( pos = header.find( "White" ) ) != string::npos ) || ( ( pos =  header.find( "white" ) ) != string::npos ) ) {
						header.erase( 0, 5 + pos ); // remove Result....
						if ( ( pos = header.find( "\"" ) ) != string::npos ) {
							header.erase( 0, 1 + pos );
							pos = header.find( "\"" );
							PlayerWhite = header.substr( 0, pos );
						}
					}
					if ( ( ( pos = header.find( "Black" ) ) != string::npos ) || ( ( pos =  header.find( "black" ) ) != string::npos ) ) {
						header.erase( 0, 5 + pos ); // remove Result....
						if ( ( pos = header.find( "\"" ) ) != string::npos ) {
							header.erase( 0, 1 + pos );
							pos = header.find( "\"" );
							PlayerBlack = header.substr( 0, pos );
						}
					}
				}
				header = "";
				continue;
			}
			
			if( brackets >  0 ) { // inside of header fields
				if ( c != ' ' && c != 10 && c != 13 && c != '[' && c != ']' ) header += c;
			} else header = "";

			if( brackets == 0 ) { // NOT inside of header fields
				if ( skipoldgame ) continue;
				if ( c == '.' ) return true;
			}
		}
	}

	return false;
}

bool PGNParser::getNextMove( Move** PGNMove, int *rating )
{
	Moves.ComputeLegalMoves( &PGNBoard );
	moveNo++;
	string nextMove = "";
	char toCoord[ 80 ];
	char c;
	do {
		if ( !PGNFile->read( &c, 1 ) ) c = ' ';
		if ( c != EOF ) {
			if ( c == '{' ) { brackets2++; continue; }
			if ( c == '}' ) { brackets2--; continue; }
			if ( c == '[' ) {
				brackets++;
				return false; 
			}
			if ( c == ']' ) {
				brackets--; 
				cerr << "Error found in PGNParser::getNextMove...";
				exit( 0 );
				return false;
			}
			
			if ( ( brackets != 0 ) || ( brackets2 != 0 ) ) 
				continue;

			if ( ( c == 10 || c == 13 ) && nextMove.length() > 0 && nextMove[ nextMove.length()-1 ] == '.' ) 
				continue;

			if ( c != ' ' && c != 10 && c != 13 ) {
				nextMove += c;
			} else {
				if ( nextMove.length() >= 1 && nextMove[ nextMove.length()-1 ] == '.' ) {
					nextMove = "";
					continue;
				}
				if ( nextMove == "" ) 
					continue;
				int pos;
				if ( ( pos = nextMove.find( "." ) ) != string::npos ) {
					nextMove.erase( 0, pos + 1 );
				}
				if ( nextMove == "*" )		return false;
				if ( nextMove == "1-0" )		return false;
				if ( nextMove == "0-1" )		return false;
				if ( nextMove == "1/2" )		return false;
				if ( nextMove == "1/2-1/2" )	return false;
				
				PGNtoCoordMove( nextMove.c_str(), toCoord, rating );
				if ( *rating == Move::MOVE_RATING_UNKNOWN ) {
					if ( PGNBoard.CurrentPlayer == winner ) *rating = Move::MOVE_RATING_GOOD; 
					else {
						if ( PGNBoard.CurrentPlayer == Player::SIDE_WHITE && winner == SIDE_BLACK ) *rating = Move::MOVE_RATING_POOR;
						if ( PGNBoard.CurrentPlayer == Player::SIDE_BLACK && winner == SIDE_WHITE ) *rating = Move::MOVE_RATING_POOR;
					}
				}

				int source, dest;
				Board::CoordToSquare( toCoord, &source, &dest );
				Move* theMove;
				Moves.ComputeLegalMoves( &PGNBoard );
				theMove = Moves.FindMoveForSquares( source, dest );
				if ( theMove == NULL ) {
					cerr << endl <<">> Fehler in PGNParser::getNextMove MatchNo: " << matchNo << " MoveNo:" << moveNo << endl;
					exit( 0 );
					return false;
				}

				if ( theMove->MoveType == Move::MOVE_PROMOTION_BISHOP ||
					theMove->MoveType == Move::MOVE_PROMOTION_KNIGHT ||
					theMove->MoveType == Move::MOVE_PROMOTION_QUEEN  ||
					theMove->MoveType == Move::MOVE_PROMOTION_ROOK )
					theMove->MoveType = PGNParser::getPromotionType( toCoord[ 4 ] );

				if ( theMove->MoveType == Move::MOVE_PROMOTION_BISHOP + Move::MOVE_CAPTURE_ORDINARY ||
					theMove->MoveType == Move::MOVE_PROMOTION_KNIGHT + Move::MOVE_CAPTURE_ORDINARY ||
					theMove->MoveType == Move::MOVE_PROMOTION_QUEEN  + Move::MOVE_CAPTURE_ORDINARY ||
					theMove->MoveType == Move::MOVE_PROMOTION_ROOK   + Move::MOVE_CAPTURE_ORDINARY ) 
					theMove->MoveType = PGNParser::getPromotionType( toCoord[ 4 ] ) + Move::MOVE_CAPTURE_ORDINARY;
				
				*PGNMove = theMove; 

				if ( !PGNBoard.ApplyMove( theMove ) ) {
					cerr << endl <<">> Fehler in PGNParser::getNextMove ( 2 ) MatchNo: " << matchNo << " MoveNo:" << moveNo << endl;
					exit( 0 );
					return false;
				}
				
				return true;
			}
		}
	} while ( c != EOF );

	return false;
}



// PGNtoCoordMove
// http://www.stanford.edu/~jjshed/chess/
// modified....
#define GETFILE( i )	( i % 8 )		
#define GETRANK( i )	( 7-( i/8 ) ) 
#define GETCOORD( x, y )		( x + ( ( 7-y )*8 ) )


void PGNParser::PGNtoCoordMove( const char *pgn, char *coord, int *rating )
{
	const int WHITE = SIDE_WHITE;
	const int BLACK = SIDE_BLACK;
	const int PGN_MOVE_LEN = 8;
	int side = PGNBoard.CurrentPlayer;
	int color;
	*rating = Move::MOVE_RATING_UNKNOWN;

	// modifies pgn, taking the " + " off of a checking move, if it's there
	// Coord buffer size must be at least PGN_MOVE_LEN
	int sqr;
	char *ptr;
	char mover[ PGN_MOVE_LEN ];
	char tcoord[ 2 ];
	int i, rank, file, piece;
	memset( coord,'\0',PGN_MOVE_LEN );
	if( ( ptr = strchr( pgn,' + ' ) ) != NULL ) *ptr = '\0';
	if( ( ptr = strchr( pgn, '#' ) ) != NULL ) *ptr = '\0';

	if( ( ptr = strstr( pgn, "!?\0" ) ) != NULL ) *ptr = '\0';
	if( ( ptr = strstr( pgn, "?!\0" ) ) != NULL ) *ptr = '\0';

	if( ( ptr = strchr( pgn, '!' ) ) != NULL ) {
		*rating = Move::MOVE_RATING_GOOD;
		*ptr = '\0';
	}

	if( ( ptr = strchr( pgn, '?' ) ) != NULL ) {
		*rating = Move::MOVE_RATING_GOOD;
		*ptr = '\0';
	}

	// move is a pawn move
	if( strlen( pgn ) == 2 ) {
		sqr = CoordToSqr( pgn );
		coord[ 0 ] = pgn[ 0 ];
		coord[ 2 ] = pgn[ 0 ];
		coord[ 3 ] = pgn[ 1 ];
		if( side == WHITE ) {
			if( PGNBoard.FindWhitePiece( sqr + 8 ) == Board::WHITE_PAWN ) {
				coord[ 1 ] = pgn[ 1 ]-1;
			} else {
				coord[ 1 ] = pgn[ 1 ]-2;
			}
		} else {
			if( PGNBoard.FindBlackPiece( sqr-8 ) == Board::BLACK_PAWN ) {
				coord[ 1 ] = pgn[ 1 ] + 1;
			} else {
				coord[ 1 ] = pgn[ 1 ] + 2;
			} 
		}
		return;
	} 
	
	// 0-0 is non-stardard, but I'm trying to provide robust support here. 
	// Some people seem unable to hold to the PGN standard.	
	if( !strcmp( pgn, "O-O" ) || !strcmp( pgn, "0-0" ) ) { 
		if( side == WHITE ) {
			memcpy( coord,"e1g1", 5 );
		} else {
			memcpy( coord, "e8g8", 5 );
		}
		return;
	}
	if( !strcmp( pgn, "O-O-O" ) || !strcmp( pgn, "0-0-0" ) ) {
		if( side == WHITE ) {
			memcpy( coord, "e1c1",5 );
		} else {
			memcpy( coord, "e8c8",5 );
		}
		return;
	}

	// Moves that can combo 
	// Like exf7=Q
	sqr = -1;
	memset( mover,'\0',PGN_MOVE_LEN );
	// hunt for the destination sqr

	if( ( ptr = strchr( pgn, 'x' ) ) != NULL ) {
		// This move is a capture!
		sqr = CoordToSqr( ptr + sizeof( char ) );
		memcpy( mover,pgn,ptr-pgn );
	}

	if( ( ptr = strchr( pgn, '=' ) ) != NULL ) {
		// This move is a promotion!
		coord[ 4 ] = *( ptr + sizeof( char ) );        // promote to
	    sqr = CoordToSqr( pgn + strlen( pgn )-4 ); // dest sqr 
		SqrToCoord( sqr, tcoord );
		memcpy( coord + 2 ,tcoord,2 );
		coord[ 0 ] = pgn[ 0 ];
		coord[ 5 ] = '\0';
		if( side == WHITE ) {
			coord[ 1 ] = coord[ 3 ]-1;
		} else {
			coord[ 1 ] = coord[ 3 ] + 1;
		}
		return;
	}

	// Catch cases when a pawn capture = promotion
	// This appears to be a regular piece move
	if( sqr == -1 ) {
		sqr = CoordToSqr( pgn + strlen( pgn )-2 ); // dest sqr is in last two positions
	}
	if( mover[ 0 ] == '\0' ) {
		memcpy( mover, pgn, strlen( pgn )-2 );
	}


	// mover now has piece to move in it
	// sqr now has the destination square in it
	if( islower( mover[ 0 ] ) ) {
		// this is a pawn move
		if( strlen( mover ) == 1 ) {
			memcpy( coord + 2, pgn + strlen( pgn )-2, 2 ); // dest sqr is in last two positions
			coord[ 0 ] = pgn[ 0 ];

			if( side == WHITE ) {
				coord[ 1 ] = coord[ 3 ]-1;
			} else {
				coord[ 1 ] = coord[ 3 ] + 1;
			}
			return;
		} else {
			cerr << "watt is datt ?" << endl;
			exit( 0 );
		}
		for( i = 0; i < 8; i++ ) {
			piece = PGNBoard.FindPiece( GETCOORD( mover[ 0 ]-'a',i ), &color );

			if( piece == Board::PAWN && color == side ) {
				if ( !Moves.isAttacking( &PGNBoard, GETCOORD( mover[ 0 ]-'a',i ), sqr ) ) 
					continue;  
				Move* checkMove;
				if ( ( checkMove = Moves.FindMoveForSquares( GETCOORD( mover[ 0 ]-'a',i ), sqr ) ) == NULL ) 
					continue;
				Board checkLegal;
				checkLegal.Clone( &PGNBoard );
				if ( !checkLegal.ApplyMove( checkMove ) ) 
					continue;
				MoveListGenerator checkMoveList;
				if ( !checkMoveList.ComputeLegalMoves( &checkLegal ) ) // checks wheather king in check
					continue;  
				
				coord[ 0 ] = mover[ 0 ];
				coord[ 1 ] = '1' + i;
				SqrToCoord( sqr, tcoord );
				memcpy( coord + 2,tcoord,2 );
				return;
			}
		}
		
	} else {
		rank = -1;
		file = -1;
		// This is a piece move
		if( strlen( mover ) > 1 ) {
			// stupid disambiguation
			if( isdigit( mover[ 1 ] ) ) {
				rank = mover[ 1 ]-'1';
			}
			if( isdigit( mover[ 2 ] ) ) {
				rank = mover[ 2 ]-'1';
			}
			if( isalpha( mover[ 1 ] ) ) {
				file = mover[ 1 ]-'a';
			}
		}


		piece = -10;
		if( mover[ 0 ] == 'Q' ) piece = Board::QUEEN;
		if( mover[ 0 ] == 'K' ) piece = Board::KING;
		if( mover[ 0 ] == 'N' ) piece = Board::KNIGHT;
		if( mover[ 0 ] == 'B' ) piece = Board::BISHOP;
		if( mover[ 0 ] == 'R' ) piece = Board::ROOK;

		if( piece == -10 ) {
			printf( "Piece not found, PGN Error!\n" );
			return;
		}

		for( i = 0; i < 64; i++ ) {
			int boardpiece = PGNBoard.FindPiece( i, &color );
			if( color == side && boardpiece == piece ) {
				if( rank == -1 || GETRANK( i ) == rank ) {
					if( file == -1 || GETFILE( i ) == file ) {
						if ( !Moves.isAttacking( &PGNBoard, i, sqr ) ) 
							continue;
						Move* checkMove;
						if ( ( checkMove = Moves.FindMoveForSquares( i, sqr ) ) == NULL ) 
							continue;
						Board checkLegal;
						checkLegal.Clone( &PGNBoard );
						if ( !checkLegal.ApplyMove( checkMove ) ) 
							continue;
						MoveListGenerator checkMoveList;
						if ( !checkMoveList.ComputeLegalMoves( &checkLegal ) ) // checks wheather king in check
							continue;  
						SqrToCoord( i, tcoord );
						memcpy( coord,tcoord, 2 );
						SqrToCoord( sqr,tcoord );
						memcpy( coord + 2, tcoord, 2 );
						return;
					}
				}
			}
		}
	}
	printf( "ERROR!!! PGN file is ill-formed!\n" );
}

int PGNParser::CoordToSqr( const char *coord )
{
	return ( coord[ 0 ] - 'a' ) + ( 8*( 7-( coord[ 1 ]-'1' ) ) );
}

void PGNParser::SqrToCoord( int sqr, char *coord )
{
	coord[ 0 ] = 'a' + GETFILE( sqr );
	coord[ 1 ] = '1' + GETRANK( sqr );
}

int PGNParser::getPromotionType( const char c ) {
		if( c == 'Q' ) return Move::MOVE_PROMOTION_QUEEN;
		if( c == 'N' ) return Move::MOVE_PROMOTION_KNIGHT;
		if( c == 'B' ) return Move::MOVE_PROMOTION_BISHOP;
		if( c == 'R' ) return Move::MOVE_PROMOTION_ROOK;
		cerr << "Promotion not found" << endl;
		exit( 0 );
		return -1;
}

