// PlayerHuman.cpp: Implementierung der Klasse PlayerHuman.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "stdafx.h"
#include "PlayerHuman.h"


/***************************************************************************
 * Static Class members
 **************************************************************************/

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif

/***************************************************************************
 * Constructor / Destructor
 **************************************************************************/
PlayerHuman::PlayerHuman( int which ) : Player()
{
    SetSide( which );
}

PlayerHuman::~PlayerHuman()
{
}

/***************************************************************************
 * Operations
 **************************************************************************/

// Getting a move from the human player. That means the user has to type a
// move ( e.g. e2e4 ) and this function checks if its valid
bool PlayerHuman::GetMove( Board* theBoard, Move* mov )
{
    Board   Successor; 
	char	PawnPromotion;

		// Read the move from the command line
    bool ok = false;
    do	{
		if ( !getInput( mov, &PawnPromotion ) ) continue;
		if ( WinBoardConnection::changeAI == true) {
		    int otherSide = ( GetSide() + 1 ) % 2;
			WinBoardConnection::changeAI = false;
			*Output::cout << "Humen -> AI" << endl;
			Game::GetInstance()->SetPlayer( otherSide, Game::HUMANWINBOARD, "Winboard - Human" );
			Game::GetInstance()->SetPlayer( GetSide(), Game::AIWINBOARD, "Winboard - AI" );
			return Game::GetInstance()->Players[ GetSide() ]->GetMove(theBoard, mov);
		}

		if ( mov->MoveType == Move::MOVE_RESIGN )
			return true;

		// Extract the source and destination squares from the line buffer
		if ( ( mov->SourceSquare < 0 ) || ( mov->SourceSquare > 63 ) ) {
			*Output::cout << "Sorry, illegal source square " << mov->SourceSquare << "\n";
			continue;
		}
		if ( ( mov->DestinationSquare < 0 ) || ( mov->DestinationSquare > 63 ) ) {
			*Output::cout << "Sorry, illegal destination square " << mov->DestinationSquare << "\n";
			continue;
		}

		// Time to try to figure out what the move means!
		if ( theBoard->GetCurrentPlayer() == Player::SIDE_WHITE ) {
			// Is there a piece ( of the moving player ) on SourceSquare?
			// If not, abort
			mov->MovingPiece = theBoard->FindWhitePiece( mov->SourceSquare );
			if ( mov->MovingPiece == Board::EMPTY_SQUARE ) {
				*Output::cout << "Sorry, You don't have a piece at square " << mov->SourceSquare << "\n";
				continue;
			}

			// Three cases: there is a piece on the destination square ( a capture ),
			// the destination square allows an en passant capture, or it is a
			// simple non-capture move.  If the destination contains a piece of the
			// moving side, abort
			if ( theBoard->FindWhitePiece( mov->DestinationSquare ) != Board::EMPTY_SQUARE ) {
				*Output::cout << "Sorry, can't capture your own piece!" << "\n";
				continue;
			}
			mov->CapturedPiece = theBoard->FindBlackPiece( mov->DestinationSquare );
			if ( mov->CapturedPiece != Board::EMPTY_SQUARE )
				 mov->MoveType = Move::MOVE_CAPTURE_ORDINARY;
			else if ( ( theBoard->GetEnPassantPawn() == ( 1 << mov->DestinationSquare ) ) &&
					  ( mov->MovingPiece == Board::WHITE_PAWN ) ) {
					mov->CapturedPiece = Board::BLACK_PAWN;
					mov->MoveType = Move::MOVE_CAPTURE_EN_PASSANT;
			}

			// If the move isn't a capture, it may be a castling attempt
			else if ( ( mov->MovingPiece == Board::WHITE_KING ) &&
				    ( ( mov->SourceSquare - mov->DestinationSquare ) == -2 ) )
					mov->MoveType = Move::MOVE_CASTLING_KINGSIDE;
			else if ( ( mov->MovingPiece == Board::WHITE_KING ) &&
					( ( mov->SourceSquare - mov->DestinationSquare ) == 2 ) )
					mov->MoveType = Move::MOVE_CASTLING_QUEENSIDE;
			else
				mov->MoveType = Move::MOVE_NORMAL;
		} else {
			mov->MovingPiece = theBoard->FindBlackPiece( mov->SourceSquare );
			if ( mov->MovingPiece == Board::EMPTY_SQUARE ) {
				*Output::cout << "Sorry, you don't have a piece in square " << mov->SourceSquare << "\n";
				continue;
			}

			if ( theBoard->FindBlackPiece( mov->DestinationSquare ) != Board::EMPTY_SQUARE ) {
				*Output::cout << "Sorry, you can't capture your own piece in square " << mov->DestinationSquare << "\n";
				continue;
			}

			mov->CapturedPiece = theBoard->FindWhitePiece( mov->DestinationSquare );
			if ( mov->CapturedPiece != Board::EMPTY_SQUARE )
				mov->MoveType = Move::MOVE_CAPTURE_ORDINARY;
			else if ( ( theBoard->GetEnPassantPawn() == ( 1 << mov->DestinationSquare ) ) &&
					( mov->MovingPiece == Board::BLACK_PAWN ) ) {
				mov->CapturedPiece = Board::WHITE_PAWN;
				mov->MoveType = Move::MOVE_CAPTURE_EN_PASSANT;
			}
			else if ( ( mov->MovingPiece == Board::BLACK_KING ) &&
				    ( ( mov->SourceSquare - mov->DestinationSquare ) == -2 ) )
				mov->MoveType = Move::MOVE_CASTLING_KINGSIDE;
			else if ( ( mov->MovingPiece == Board::BLACK_KING ) &&
					( ( mov->SourceSquare - mov->DestinationSquare ) == 2 ) )
				mov->MoveType = Move::MOVE_CASTLING_QUEENSIDE;
			else
				mov->MoveType = Move::MOVE_NORMAL;
		}

		// Now, if the move results in a pawn promotion, we must ask the user
		// for the type of promotion!
		if ( ( ( mov->MovingPiece == Board::WHITE_PAWN ) && ( mov->DestinationSquare < 8 ) ) ||
			 ( ( mov->MovingPiece == Board::BLACK_PAWN ) && ( mov->DestinationSquare > 55 ) ) ) {
			int car = -1;
			while ( ( PawnPromotion != 'K' ) && ( PawnPromotion != 'k' ) && ( PawnPromotion != 'b' ) && ( PawnPromotion != 'B' )
				 && ( PawnPromotion != 'R' ) && ( PawnPromotion != 'r' ) && ( PawnPromotion != 'Q' ) && ( PawnPromotion != 'q' ) ) {
				*Output::cout << "Promote the pawn to [ K ]night, [ R ]ook, [ B ]ishop, [ Q ]ueen?";
				car = getchar();
			} 
			if ( ( car == 'K' ) || ( car == 'k' ) )
				mov->MoveType += Move::MOVE_PROMOTION_KNIGHT;
			else if ( ( car == 'B' ) || ( car == 'b' ) )
				mov->MoveType += Move::MOVE_PROMOTION_BISHOP;
			else if ( ( car == 'R' ) || ( car == 'r' ) )
				mov->MoveType += Move::MOVE_PROMOTION_ROOK;
			else
				mov->MoveType += Move::MOVE_PROMOTION_QUEEN;
		}

		// OK, now let's see if the move is actually legal!  First step: a check
		// for pseudo-legality, i.e., is it a valid successor to the current
		// board?
		Validator.ComputeLegalMoves( theBoard );
		if ( !Validator.Find( mov ) ) {
			*Output::cout << "Sorry, this move is not in the pseudo-legal list: " << "\n";
			mov->Print();
			Validator.Print();
			continue;
		}

		// If pseudo-legal, then verify whether it leaves the king in check
		Successor.Clone( theBoard );
		Successor.ApplyMove( mov );
		if ( !Validator.ComputeLegalMoves( &Successor ) ) {
			*Output::cout << "Sorry, this move leaves your king in check: " << "\n";
			mov->Print();
			continue;
		}

		// If we have made it here, we have a valid move to play!
		ok = true;

    } while ( !ok );

    return true;
}


// waiting for stdin of human player
bool PlayerHuman::getInput( Move *InputMove, char* PawnPromotion )
{
    *Output::cout << "Your move, " << PlayerString << " ( e.g. e2e4 ) ?  ";
	*PawnPromotion = -1;

	char ch;
	string line = "";
	do {
		cin.get( ch );
		switch ( ch ) {
			case '\n':	break;
			default:    line += tolower( ch );
		}
	} while ( ch != '\n' );

	if ( line.length() < 4 ) {
		*Output::cout << "Sorry, no legal syntax !\n";
		return false;
	}

	
	if ( ( line.substr( 0,4 ) == "last" ) || ( line.substr( 0,4 ) == "back" ) || ( line.substr( 0,4 ) == "load" ) || ( line.substr( 0,4 ) == "save" ) || ( line.substr( 0,4 ) == "stor" ) ) {
		
		string par = "";
		if ( line.find(' ' ) != string::npos ) par = line.substr( line.find(' ' ) + 1 );
		while ( par[ 0 ] == ' ' ) par.erase( 0, 1 );

		if ( line.substr( 0,4 ) == "load" )
			Game::GetInstance()->load( par );
		
		if ( ( line.substr( 0,4 ) == "save" ) || ( line.substr( 0,4 ) == "stor" ) )
			Game::GetInstance()->save( par );
		
		if ( ( line.substr( 0,4 ) == "last" ) || ( line.substr( 0,4 ) == "back" ) )
			Game::GetInstance()->restoreLastMove();
		
		getInput( InputMove, PawnPromotion );
		return true;
	}

	if ( line.substr( 0,4 ) == "resi" || line.substr( 0,4 ) == "quit" || line.substr( 0,4 ) == "exit" ){
		InputMove->MoveType = Move::MOVE_RESIGN;
	} else {
		Board::CoordToSquare( line, &InputMove->SourceSquare, &InputMove->DestinationSquare );
		// is there a pawn promotion ?
		if ( line.length() >= 5 ) *PawnPromotion = line.c_str()[ 4 ]; 
	}

	return true;
}
