// StringQueue.cpp: Implementierung der Klasse StringQueue.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "StringQueue.h"

/***************************************************************************
 * Static Class members
 **************************************************************************/

const int StringQueue::STRINGQUEUE_MAXSIZE = 40;

/***************************************************************************
 * Constructor / Destructor
 **************************************************************************/

StringQueue::StringQueue()
{
	size = 0;
	strings = new string[ STRINGQUEUE_MAXSIZE ];
	InitializeCriticalSection( &criticalsection );
}

StringQueue::~StringQueue()
{
	delete [] strings;
}

/***************************************************************************
 * Operations
 **************************************************************************/

int StringQueue::GetSize()
{
	EnterCriticalSection( &criticalsection );
	return size;
	LeaveCriticalSection( &criticalsection );
}

bool StringQueue::IsEmpty()
{
	EnterCriticalSection( &criticalsection );
	if ( size == 0 ) { LeaveCriticalSection( &criticalsection ); return true; } 
		else { LeaveCriticalSection( &criticalsection ); return false; }
}

void StringQueue::Clear()
{
	EnterCriticalSection( &criticalsection );
	size = 0;
	LeaveCriticalSection( &criticalsection );
}

bool StringQueue::Push( const string& String )
{
	EnterCriticalSection( &criticalsection );

	if ( ( size + 1 ) >= STRINGQUEUE_MAXSIZE ) {
		LeaveCriticalSection( &criticalsection );
		return false;
	}

	strings[ size++ ] = String;

	LeaveCriticalSection( &criticalsection );
	return true;
}

bool StringQueue::Get( string *String )
{
	if ( String == NULL ) return false;

	EnterCriticalSection( &criticalsection );
	if ( size == 0 ) {
		LeaveCriticalSection( &criticalsection );
		return false;
	}

	*String = strings[ 0 ];

	LeaveCriticalSection( &criticalsection );
	return true;
}

bool StringQueue::Pop( string *String )
{
	if ( String == NULL ) return false;
	
	EnterCriticalSection( &criticalsection );
	if ( size == 0 ) {
		LeaveCriticalSection( &criticalsection );
		return false;
	}

	*String = strings[ 0 ];

	RemoveFirst();

	LeaveCriticalSection( &criticalsection );
	return true;
}

bool StringQueue::RemoveFirst() {
	EnterCriticalSection( &criticalsection );
	if ( size == 0 ) {
		LeaveCriticalSection( &criticalsection );
		return false;
	}

	string oldstring = strings[ 0 ];

	for ( int i = 0; i < ( size-1 ); i++ )
		strings[ i ] = strings[ i + 1 ];

	strings[ size-1 ] = oldstring;

	size--;

	LeaveCriticalSection( &criticalsection );
	return true;
}
