// TranspositionTable.cpp: Implementierung der Klasse TranspositionTable.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TranspositionTable.h"

/***************************************************************************
 * Static Class members
 **************************************************************************/

const int TranspositionTable::TABLE_SIZE = 8*8*2048*6; 

/***************************************************************************
 * Constructor / Destructor
 **************************************************************************/

TranspositionTable::TranspositionTable(int Side)
{
	Table = ( TranspositionEntry** ) malloc( TABLE_SIZE * sizeof( TranspositionEntry* ) );
	for ( int i = 0; i < TABLE_SIZE; i++ )
		Table[ i ] = new TranspositionEntry();

	this->Side = Side;
}

TranspositionTable::~TranspositionTable()
{

}

/***************************************************************************
 * Operations
 **************************************************************************/

// bool LookupBoard( Board theBoard, Move theMove )
// Verify whether there is a stored evaluation for a given board.
// If so, return TRUE and copy the appropriate values into the
// output parameter
bool TranspositionTable::LookupBoard( Board* theBoard, Move* theMove )
{
    // Find the board's hash position in Table
    int key = abs( theBoard->HashKey() % TABLE_SIZE );
    TranspositionEntry* entry = Table[ key ];

    // If the entry is an empty placeholder, we don't have a match
    if ( entry->theEvalType == TranspositionEntry::NULL_ENTRY )
		return false;

    // Check for a hashing collision!
    if ( entry->theLock != theBoard->HashLock() ) {
		return false;
	}

    // Now, we know that we have a match!  Copy it into the output parameter
    // and return
    theMove->MoveEvaluation		= entry->theEval;
    theMove->MoveEvaluationType = entry->theEvalType;
    theMove->SearchDepth		= entry->theDepth;
	theMove->SourceSquare		= entry->theSourceSquare;
	theMove->DestinationSquare	= entry->theDestinationSquare;
	theMove->MovingPiece        = entry->theNode; // <<<--- hack
    return true;
}

bool TranspositionTable::LookupBoard( Board* theBoard, TranspositionEntry** theEntry )
{
    // Find the board's hash position in Table
    int key = abs( theBoard->HashKey() % TABLE_SIZE );
    TranspositionEntry* entry = Table[ key ];

	*theEntry = entry;

    // If the entry is an empty placeholder, we don't have a match
    if ( entry->theEvalType == TranspositionEntry::NULL_ENTRY )
		return false;

    // Check for a hashing collision!
    if ( entry->theLock != theBoard->HashLock() ) {
		return false;
	}

    // Now, we know that we have a match! 
    return true;
}

	
bool TranspositionTable::PrintToFile( const char *filename, bool simplified )
{
	FILE *file;
	
	if ( ( file = fopen( filename, "w" ) ) == NULL )
		return false;
	
	int NumEntries = 0;
	for ( int i = 0; i < TABLE_SIZE; i++ ) {
		if ( Table[ i ]->theEvalType != TranspositionEntry::NULL_ENTRY ) {
			NumEntries++;
			if ( simplified )
				fprintf( file, " % d, % d, % d, % d, % d, % d\n", 
								i, Table[ i ]->theDepth, Table[ i ]->timeStamp, Table[ i ]-> theEval, Table[ i ]->theEvalType, Table[ i ]->theLock );
				else
				fprintf( file, "# % 7d: depth = % 2d, time = % 3d, eval = % 5d, evaltype = % d, lock = % 10u\n", 
								i, Table[ i ]->theDepth, Table[ i ]->timeStamp, Table[ i ]-> theEval, Table[ i ]->theEvalType, Table[ i ]->theLock );
		}
	}
	fprintf( file, "total nuber of entries: %d ( used %d %% )", NumEntries, NumEntries * 100 / TranspositionTable::TABLE_SIZE );
	fclose( file );
	return true;
}


// public StoreBoard( theBoard, eval, evalType, depth, timeStamp )
// Store a good evaluation found through alphabeta for a certain board position
bool TranspositionTable::StoreBoard( Board* theBoard, int eval, int evalType, int depth, int timeStamp, Move* mov, int node )
{
	int key = abs( theBoard->HashKey() % TABLE_SIZE );

    // Would we erase a more useful ( i.e., higher ) position if we stored this
    // one?  If so, don't bother!
/*
    if ( ( Table[ key ]->theDepth > depth ) &&
		 ( Table[ key ]->theEvalType != TranspositionEntry::NULL_ENTRY ) &&
         ( Table[ key ]->timeStamp >= timeStamp ) )
		return true;
*/
    // And now, do the actual work
    Table[ key ]->theLock = theBoard->HashLock();
	Table[ key ]->theNode = node;
    Table[ key ]->theEval = eval;
    Table[ key ]->theDepth = depth;
    Table[ key ]->theEvalType = evalType;
    Table[ key ]->timeStamp = timeStamp;
	Table[ key ]->theSourceSquare = mov->SourceSquare;
	Table[ key ]->theDestinationSquare = mov->DestinationSquare;
    return true;
}

istream& TranspositionTable::read( istream &is )
{

	for ( int i = 0; i < TABLE_SIZE; i++ )
		Table[ i ]->read( is );

	return is;
}


ostream& TranspositionTable::write( ostream &os ) const
{
	for ( int i = 0; i < TABLE_SIZE; i++ )
		Table[ i ]->write( os );

	return os;
}

void TranspositionTable::Clear()
{
	for ( int i = 0; i < TABLE_SIZE; i++ ) {
		Table[ i ]->theEvalType = TranspositionEntry::NULL_ENTRY;
		Table[ i ]->theDepth  = 0;
		Table[ i ]->theEval   = 0;
		Table[ i ]->theLock   = 0;
		Table[ i ]->timeStamp = -1; 
	}
}

void TranspositionTable::SafeClear()
{
	for ( int i = 0; i < TABLE_SIZE; i++ ) {
		if ( Table[ i ]->theEvalType = Move::EVALTYPE_ACCURATE ) continue;
		Table[ i ]->theEvalType = TranspositionEntry::NULL_ENTRY;
		Table[ i ]->theDepth  = 0;
		Table[ i ]->theEval   = 0;
		Table[ i ]->theLock   = 0;
		Table[ i ]->timeStamp = -1; 
	}
}



int TranspositionTable::getNumEntries(int depth)
{
	int NumEntries = 0;
	for ( int i = 0; i < TABLE_SIZE; i++ ) {
		if ( Table[ i ]->theEvalType != TranspositionEntry::NULL_ENTRY ) {
			if ( ( depth == -1 ) || (depth == Table[ i ]->theDepth ) ) NumEntries++;
		}
	}
	return NumEntries;
}

void TranspositionTable::ShowStats()
{
	const int MAXPLY = 1000;
	
	int NumEntries = 0;
	int aDepth[ MAXPLY ];
	int aPly  [ MAXPLY ];
	int i;
	
	for ( i = 0; i < MAXPLY; i++ ) {
		aDepth[ i ] = 0;
		aPly  [ i ] = 0;
	}

	*Output::cout << endl << " S T A T S  -  TranspositionTable ";

	if ( Side == Player::SIDE_WHITE ) 
		*Output::cout << " white";
	if ( Side == Player::SIDE_BLACK ) 
		*Output::cout << " black";

	*Output::cout << endl;

	for ( i = 0; i < TABLE_SIZE; i++ ) {
		if ( Table[ i ]->theEvalType != TranspositionEntry::NULL_ENTRY ) {
			NumEntries++;
			if ( Table[ i ]->timeStamp >= 0 && Table[ i ]->timeStamp < MAXPLY ) {
				aDepth[	Table[ i ]->theDepth + MAXPLY / 2]++;
				aPly[ Table[ i ]->timeStamp  ]++;
			} else {
				cerr << " TRANSPOSITIONTABLE IS CORRUPTED ";
			}
		}
	}

	*Output::cout << "total number of entries: " << setw( 7 ) << NumEntries << " of " << setw( 7 ) << TABLE_SIZE << " ( " << ( NumEntries * 10000 / TABLE_SIZE ) / 100.0 << "% )" << endl;

/*
	*Output::cout << "- depth" << endl;

	for ( i = MAXPLY - 1; i >= 0 ; i-- ) {
		if ( aDepth[ i ] != 0 )
			*Output::cout << "  ^- " << setw( 3 ) << i - MAXPLY / 2 << ": " << setw( 7 ) << aDepth[i] << endl;
	} 

	
	*Output::cout << "- ply" << endl;
	
	for ( i = MAXPLY - 1; i >= 0; i-- ) {
		if ( aPly[ i ] != 0 )
			*Output::cout << "  ^- " << setw( 3 ) << i << ": " << setw( 7 ) << aPly[i] << endl;
	}
*/	
	return;
}

void TranspositionTable::SetSide(int Side)
{
	this->Side = Side;
}
